<?php
/**
 * API Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class ApiController extends AppController {
	var $name = 'Api';
	var $components = array('RequestHandler');

	function beforeFilter() {
//		Configure::write('debug', 1);
		if (!Configure::read("production_mode")) {
			Configure::write('debug', 3);
		}
		//$this->RequestHandler->setContent('json');
		//$this->RequestHandler->respondAs('application/json; charset=UTF-8');
		$this->layout = "api";

		if (Configure::read("production_mode")) {
			$this->validateIp();
			$this->validationCertification();
		}
	}

	function validateIp() {
		$ipList = Configure::read("api.ip");
		if (empty($ipList)) return;

		foreach ($ipList as $ip) {
			if ($ip === $_SERVER['REMOTE_ADDR']) {
				return;
			}
		}
		$this->apiErrorLog('IP addresses is not allowed.');
		$this->cakeError('jsonCannotAccess');
	}

	function validationCertification() {
		if (empty($_SERVER["SSL_CLIENT_CERT"]) || empty($_SERVER["SSL_CLIENT_VERIFY"])) {
			$this->apiErrorLog('SSL_CLIENT not set');
			if (!empty($_SERVER["SSL_CLIENT_CERT"])) {
				$this->apiErrorLog('SSL_CLIENT_CERT:'.$_SERVER["SSL_CLIENT_CERT"]);
			}
			if (!empty($_SERVER["SSL_CLIENT_VERIFY"])) {
				$this->apiErrorLog('SSL_CLIENT_VERIFY:'.$_SERVER["SSL_CLIENT_VERIFY"]);
			}
			$this->cakeError('jsonCannotAccess');
		}
		$cert = $this->getClientCert();
		$count = $this->SpHost->find('count', array('conditions' => array(
			'OR' => array('certificate' => $cert, 'certificate2' => $cert))));
		if ($count <= 0) {
			$this->apiErrorLog('not found certificate');
			$this->apiErrorLog('SSL_CLIENT_CERT='.$_SERVER["SSL_CLIENT_CERT"]);
			$this->apiErrorLog('$cert='.$cert);
			$this->cakeError('jsonCannotAccess');
		}
	}

	/**
	 * @return integer SP_HOST ID
	 */
	function getSpHost() {
		if (Configure::read("production_mode")) {
			$cert = $this->getClientCert();
			$spHost = $this->SpHost->find('first', array('fields' => array('id', 'entityid'),
				'conditions' => array('OR' => array('certificate' => $cert, 'certificate2' => $cert))));

			if (empty($spHost)) {
				$this->apiErrorLog('SpHost is empty.');
				$this->cakeError('jsonCannotAccess');
			}
			return $spHost;
		}
		$spHost = $this->SpHost->find('first', array('fields' => array('id', 'entityid'),
			'conditions' => array('name' => 'shibosuke.net')));
		return $spHost;
	}

	private function getClientCert() {
		$cert =  $_SERVER["SSL_CLIENT_CERT"];
		$cert = ereg_replace("-----BEGIN CERTIFICATE-----","",$cert);
		$cert = ereg_replace("-----END CERTIFICATE-----", '', $cert);
		$cert = ereg_replace("\n", '', $cert);
		$cert = ereg_replace("\r", '', $cert);
		$cert = ereg_replace(" ", '', $cert);
		return $cert;
	}


	function apiErrorLog($message) {
		$this->apiLog($message, 'ERROR');
	}

	function apiInfoLog($message) {
		$this->apiLog($message, 'INFO ');
	}

	private function apiLog($message, $level) {
		$addr = $_SERVER["REMOTE_ADDR"];
		$this->log('['.$addr.'] '.$level.' '.$message, 'api');
	}
}
?>
